import { race, Subject, timer } from 'rxjs';
import { first, tap } from 'rxjs/operators';
export default class Debugger {
  constructor() {
    this.breaker = new Subject();
    this.panicker = new Subject();
    this.enabled = true;
    this.lastThrottle = new Date();
  }
  /**
   * Just like `this.wait(0)` but only triggers every @ms milliseconds so that
   * it will not become a bottleneck. Put in a tight loop to make sure that the
   * UI still breathes.
   */
  async alive(ms = 250) {
    const now = new Date();
    if (now.getTime() - this.lastThrottle.getTime() < ms) {
      return;
    }
    this.lastThrottle = now;
    await this.wait(0);
  }
  async wait(options) {
    if (!this.enabled) {
      return;
    }
    else if (typeof options === 'number') {
      options = { ms: options };
    }
    else if (typeof options === 'function') {
      options = { preamble: options };
    }
    else if (typeof options === 'undefined') {
      options = {};
    }
    if (typeof options.preamble === 'function')
      options.preamble();
    const observables = [
      this.breaker,
      this.panicker.pipe(tap(() => {
        throw new Error('VM panicked!');
      })),
    ];
    if (!Number.isNaN(options.ms) && typeof options.ms !== 'undefined' && options.ms !== null) {
      observables.push(timer(options.ms));
    }
    await race(observables).pipe(first()).toPromise();
    if (typeof options.callback === 'function')
      options.callback();
  }
  resume() {
    var _a;
    (_a = this.breaker) === null || _a === void 0 ? void 0 : _a.next();
  }
  panic() {
    var _a;
    (_a = this.panicker) === null || _a === void 0 ? void 0 : _a.next();
    this.destroy();
  }
  destroy() {
    var _a, _b;
    (_a = this.breaker) === null || _a === void 0 ? void 0 : _a.complete();
    (_b = this.panicker) === null || _b === void 0 ? void 0 : _b.complete();
  }
}
