import { execute, signal, init, cleanup, panic, resume } from './executor.worker';
function runOnMainFactory(context) {
  return async (callId, fn, args = []) => context.signal(callId, await Function(`return (
        ${fn.toString()}
      )(...arguments)`).apply(context, args));
}
export default class Executor {
  constructor(context) {
    this.context = context;
    this.running = false;
  }
  destroy() {
    cleanup();
  }
  panic() {
    panic();
  }
  resume() {
    resume();
  }
  async execute(code) {
    this.running = true;
    let result;
    try {
      await init();
      result = await execute(code, runOnMainFactory(Object.assign(Object.assign({}, this.context), { signal: signal })));
    }
    finally {
      await cleanup();
    }
    this.running = false;
    return result;
  }
}
